          section   TEXT

          xdef      _Aurora_test
          xdef      _Aurora_mode
          xdef      _Aurora_cls
          xdef      _Aurora_plot

* Prototype: int Aurora_test(void);
* Returns 1 if we are running on an aurora or 0 if not.
* The test is performed as per the suggested method from Nasta.
* As we are going to do dirty things, we don't want to be interrupted by another
* job during the test, so we must do it in supervisor mode.

_Aurora_test
          TRAP      #0                            Enter supervisor mode.
          MOVEQ     #1,D0                         Assume aurora is there.
          LEA       $20000,A0                     Base of first QL screen.
          MOVE.L    (A0),D1                       Save content of first 4 bytes.
          LEA       $4C0000,A1                    Base of Aurora screen.
          MOVE.L    (A1),D2                       Save content of first 4 bytes.
          CLR.L     (A1)                          Clear the first 4 bytes.
          MOVE.L    #'Test',(A0)                  Write a test string in QL scr.
          CMPI.L    #'Test',(A1)                  Data tranfered to Aurora scr ?
          BEQ.S     AURTEST1                      Aurora found.
          MOVEQ     #0,D0                         Aurora not found !
          MOVE.L    D2,(A1)                       | Restore the contents of the
AURTEST1  MOVE.L    D1,(A0)                       |  test addresses.
          ANDI.W    #$7FF,SR                      Switch back to user mode.
          RTS                                     Return.

          NOP                                     This may sound weird but this
*                                                 useless NOP is needed !  There
*                                                 is apparently a problem at
*                                                 link time and this problem
*                                                 makes POV crash on SGC BEFORE
*                                                 any Aurora function is called.
*                                                 If you change this source, you
*                                                 may have to remove this NOP or
*                                                 even to had another one IOT
*                                                 make POV to work on SGC (the
*                                                 problem seems related to the
*                                                 number of words held in the
*                                                 final Aurora_rel file...).

* Prototype: void Aurora_mode(short mode);

_Aurora_mode
          MOVE.W    4(SP),D0                      Requested mode.
          LEA       $18043,A0                     Control register address.
          MOVE.B    D0,(A0)                       Set the EMCR.
          RTS                                     Return.

* Prototype: void Aurora_cls(void);
* Clears the screen memory (all pixels set to black).

_Aurora_cls
          LEA       $4C0000,A0                    Screen memory address.
          MOVE.W    #3839,D1                      DBRA counter.
          MOVEQ     #0,D0                         Colour = black.
AURCLS1   MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |  Clears 64 bytes of
          MOVE.L    D0,(A0)+                      |   memory.
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          MOVE.L    D0,(A0)+                      |
          DBRA      D1,AURCLS1                    Do it 3840 times...
          RTS                                     Return.

* Prototype: int Aurora_plot(short mode, short x, short y, short Green,
*                            short Red, short Blue);

_Aurora_plot
          MOVEM.L   D3-D6,-(SP)                   Save the context.
          LEA       $4C0000,A0                    Screen memory address.
          LEA       20(SP),A1                     Pointer on first parameter.
          MOVE.W    (A1)+,D0                      Display mode (not used yet).
          MOVE.W    (A1)+,D1                      X coordinate.
          MOVE.W    (A1)+,D2                      Y coordinate.
          CMPI.W    #512,D1                       Check for impossible x coord.
          BPL.L     AURPLOT1                      X > 511 so return error
          CMPI.W    #480,D2                       Check for impossible y coord.
          BPL.L     AURPLOT1                      Y > 479 so return error
          MOVEQ     #0,D4                         Clear all 32 bits of D4
          MOVE.W    D2,D4                         Row ...
          MULU      #512,D4                       Multiply by row size
          ADD.L     D1,D4                         Add the column offset
          MOVE.W    (A1)+,D1                      Green color value (8 bits)
          LSR.W     #5,D1                         Strip off the 5 L.S. bits
          MOVE.W    (A1)+,D2                      Red color value (8 bits)
          LSR.W     #5,D2                         Strip off the 5 L.S. bits
          MOVE.W    (A1)+,D3                      Blue color value (8 bits)
          LSR.W     #5,D3                         Strip off the 5 L.S. bits

* Now check for conflict in
* Red and Blue color

          TST.W     D3                            If Blue is 0 then no problem.
          BNE.S     AURPLOT2                      Else go check if Red == 0.
          MOVE.W    D2,D5                         Use the Red L.S. bit for RB0.
          ANDI.W    #1,D5                         L.S. bit only (RB0).
          BRA.S     AURPLOT3                      Color maping is OK.
AURPLOT2  TST.W     D2                            If Red is 0 then no problem.
          BNE.S     AURPLOT4                      Maybe Red&Blue have same LSB ?
          MOVE.W    D3,D5                         Use the Blue L.S. bit for RB0
          ANDI.W    #1,D5                         L.S. bit only (RB0).
          BRA.S     AURPLOT3                      Color maping is OK.
AURPLOT4  MOVE.W    D2,D5                         Red value.
          ANDI.W    #1,D5                         Bit#0 only.
          MOVE.W    D3,D6                         Blue value.
          ANDI.W    #1,D6                         Bit#0 only.
          CMP.W     D5,D6                         Are they equal ?
          BEQ.S     AURPLOT3                      Yes - Color mapping is OK.
          MOVEQ     #1,D5                         Conflict: we chose to set RB0.
                                                  
* Now we build the final value

AURPLOT3  BTST      #0,D1                         Bit#0 for green ...
          BEQ.S     AURPLOT5                      (No need to set anything)
          BSET      #1,D5                         ... is bit#1 in D5
AURPLOT5  BTST      #1,D1                         Bit#1 for green ...
          BEQ.S     AURPLOT6                      (No need to set anything)
          BSET      #4,D5                         ... is bit#4 in D5
AURPLOT6  BTST      #2,D1                         Bit#2 for green ...
          BEQ.S     AURPLOT7                      (No need to set anything)
          BSET      #7,D5                         ... is bit#7 in D5
AURPLOT7  BTST      #1,D2                         Bit#1 for red ...
          BEQ.S     AURPLOT8                      (No need to set anything)
          BSET      #3,D5                         ... is bit#3 in D5
AURPLOT8  BTST      #2,D2                         Bit#2 for red ...
          BEQ.S     AURPLOT9                      (No need to set anything)
          BSET      #6,D5                         ... is bit#6 in D5
AURPLOT9  BTST      #1,D3                         Bit#1 for blue ...
          BEQ.S     AURPLOT10                     (No need to set anything)
          BSET      #2,D5                         ... is bit#2 in D5
AURPLOT10 BTST      #2,D3                         Bit#2 for blue ...
          BEQ.S     AURPLOT11                     (No need to set anything)
          BSET      #5,D5                         ... is bit#5 in D5

AURPLOT11 MOVE.B    D5,0(A0,D4.L)                 Set the pixel.
          MOVEQ     #0,D0                         No error.
AURPLOT0  MOVEM.L   (SP)+,D3-D6                   Restore the context.
          RTS
AURPLOT1  MOVEQ     #-4,D0                        "Out of range" error code
          BRA.S     AURPLOT0                      And exit

          end
